{*******************************************************************************
* Unit      : TSLockPK Demostration program "LockPK"                           *
*             Uses SLockPK in the "PublicKey" registration mode                *
*                                                                              *
********************************************************************************
* Purpose   : Demonstration/Example program for the SLockPK component          *
*             This program is intended to provide SLockPK users with ideas     *
*             and examples how to use the SLockPK component to achieve         *
*             successful results. You are at liberty to use sections of this   *
*             example code in your applications when you have purchased a      *
*             license for SLockPK                                              *
*                                                                              *
*             Note that this application is a fairly advanced example, and     *
*             similar results can be achieved with much less code if you       *
*             wish. See the other example projects for details how to do this. *
********************************************************************************
* Copyright : This software  is copyright TSM Inc. 1998-1999                   *
*             This source code may not be distributed to third parties in      *
*             or in part without the written permission of TSM Inc.            *
*             All rights reserved. Liability limited to replacement of         *
*             this original source code in the case of loss or damage because  *
*             the use or misuse of this software.                              *
********************************************************************************
* Version   : 14.05.98  - 1.00  Original Version                               *
*             22.04.99  - 2.00  PK version "SLockPK"                           *
********************************************************************************
* Switches  : None                                                             *
*******************************************************************************}
unit Slockfrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, registry, RegFrm, ExtCtrls, Menus, Slock;

type
  TfrmMain = class(TForm)
    btnRegister: TButton;
    grpStatus: TGroupBox;
    lblStatus: TLabel;
    lblStatusDisp: TLabel;
    lblDaysLeft: TLabel;
    lblDaysLeftDisp: TLabel;
    lblStartsLeftDisp: TLabel;
    lblExpiryDateDisp: TLabel;
    lblProtType: TLabel;
    lblProtTypeDisp: TLabel;
    lblStartsLeft: TLabel;
    lblExpiryDate: TLabel;
    lblRegisteredTo: TLabel;
    lblRegisteredToDisp: TLabel;
    lblSerialNumber: TLabel;
    lblSerialNumberDisp: TLabel;
    bvlStatus: TBevel;
    lblGraceStartsLeft: TLabel;
    lblGraceStartsLeftDisp: TLabel;
    mnuMainMenu: TMainMenu;
    mnmFile: TMenuItem;
    mnpExit: TMenuItem;
    mnmRegister: TMenuItem;
    mnpRegNow: TMenuItem;
    mnmFunction: TMenuItem;
    mnpNormFunc: TMenuItem;
    btnExit: TButton;
    lblVersionLabel: TLabel;
    lblVersionDisp: TLabel;
    lblCertExpireDisp: TLabel;
    lblCertExpire: TLabel;
    btnLoadCertificate: TButton;
    mnpLoadCertificate: TMenuItem;
    Slock1: TSlock;
    mnmDemo: TMenuItem;
    mnpDemoFunction: TMenuItem;
    lblUser1Label: TLabel;
    lblUser2Label: TLabel;
    lblUser1: TLabel;
    lblUser2: TLabel;
    mnmHelp: TMenuItem;
    mnpGettingHelp: TMenuItem;
    procedure Slock1FirstStart(Sender: TObject);
    procedure Slock1ClockMovedBack(Sender: TObject);
    procedure Slock1Reminder(Sender: TObject);
    procedure Slock1WrongUnlockCode(Sender: TObject);
    procedure Slock1Expired(Sender: TObject);
    procedure btnRegisterClick(Sender: TObject);
    procedure Slock1Register(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Slock1RegInfoChanged(Sender: TObject);
    procedure Slock1InGracePeriod(Sender: TObject);
    procedure Slock1RegWriteErr(Sender: TObject);
    procedure mnpNormFuncClick(Sender: TObject);
    procedure mnpExitClick(Sender: TObject);
    procedure mnpRegNowClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure Slock1Extend(Sender: TObject);
    procedure Slock1TimedOut(Sender: TObject);
    procedure Slock1RegInfoTamper(Sender: TObject);
    procedure Slock1NoInitCert(Sender: TObject);
    procedure Slock1CertAccept(Sender: TObject);
    procedure Slock1CertReject(Sender: TObject);
    procedure btnLoadCertificateClick(Sender: TObject);
    procedure Slock1CertExpired(Sender: TObject);
    procedure mnpLoadCertificateClick(Sender: TObject);
    procedure mnpDemoFunctionClick(Sender: TObject);
    procedure Slock1TrialInfoTamper(Sender: TObject);
    procedure Slock1Reset(Sender: TObject);
    procedure Slock1Terminate(Sender: TObject);
    procedure mnpGettingHelpClick(Sender: TObject);
  private
    procedure UpdateStatus;
    function  GetPKPart3: string;
  end;

var
  frmMain: TfrmMain;

implementation

uses frmInit, RegFrm1;

const
     // this is the first section of the public key. See the comments under
     // "FormCreate" for an explanation
     LIT_TEMP_PUBKEY_PART1 = '++11Ik:W7gOjEFPXdezH';

{$R *.DFM}

{*******************************************************************************
* Procedure : FormCreate                                                       *
********************************************************************************
* Purpose   : We check the current status of the protection in the form create *
*             of the main form. (Slock1.CheckProtection) The internal state of *
*             Slock is set, and we then use 'UpdateStatus' to read the status  *
*             of Slock out into the status display                             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.FormCreate(Sender: TObject);
const
     // this is the second part of the public key
     LIT_TEMP_PUBKEY_PART2 = '1gRytPR0VJwPSOUgS-R8QN5J4J9hCBRL6DDb+';
var
     TempPublicKey: string;
begin
     // in this example the private key is built up from individual sections of
     // the key which are distributed throughout the software package. This is
     // recommended to stop attackers simply patching the executable to replace
     // your key with a key of their own.

     // the key is created using several different methods:
     //   1) the first section of the key is retrieved from a global constant
     //   2) the second section is retrieved from a local constant
     //   3) the third section is retrieved from a function and is *reversed*!!
     //   4) retrieved from an inline literal
     // you can of course add your own variations to this. Put the functions in
     // different procedures! (Do some during the initialisation!) Rotate the
     // string! XOR with a counter! Change the case!

     // build the public key
     TempPublicKey := LIT_TEMP_PUBKEY_PART1 +
                      LIT_TEMP_PUBKEY_PART2 +
                      GetPKPart3 +
                      'M1HN50dXMRLZfmxKhAcUpaFWbiBA+';

     // load the public key
     SLock1.LoadPKKey(TempPublicKey);

     // Show the version of the SLock component we are using
     lblVersionDisp.Caption := SLock1.GetVersion;

     // check the protection of the Slock component
     SLock1.CheckProtection;

     // Update the status display and the state of the Buttons/Menupoints
     UpdateStatus;
end; {FormCreate}

{*******************************************************************************
* Procedure : GetPKPart3                                                       *
********************************************************************************
* Purpose   : Returns a part of the public key, reversing the byte order as it *
*             does so. See "FormCreate" for an explaination.                   *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
function TfrmMain.GetPKPart3: string;
const
     LIT_TEMP_PUBKEY_PART3_REVERSE = 'qlyWGpb';
var
     i: integer;
begin
     // initialise the result
     Result := '';

     // reverse the key section and pass back
     for i := Length(LIT_TEMP_PUBKEY_PART3_REVERSE) downto 1 do
     begin
          Result := Result + LIT_TEMP_PUBKEY_PART3_REVERSE[i];
     end; {for i}
end; {TfrmMain.GetPKPart3}

{*******************************************************************************
* Procedure : UpdateStatus                                                     *
********************************************************************************
* Purpose   : Loads the internal status of the Slock component into the status *
*             display. Also sets the state of the menus and buttons            *
*                                                                              *
*             You normally do not need a function such as this, but for the    *
*             purposes of showing you how SLockPK works this is required.      *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.UpdateStatus;
begin
     // load the status display from the Status property
     case Slock1.Status of
     stNotExpired:
          lblStatusDisp.Caption := 'Not Expired';
     stExpired:
          lblStatusDisp.Caption := 'Expired';
     stGrace:
          lblStatusDisp.Caption := 'Grace Period';
     stRegistered:
          lblStatusDisp.Caption := 'Registered';
     stDemo:
          lblStatusDisp.Caption := 'Registered Demo';
     end;

     // show the certificate expiry date
     lblCertExpire.Caption := FormatDateTime('dd/mm/yyyy', SLock1.CertExpire);

     // load the days/starts left display
     case Slock1.ProtectionType of
     ptStartCount:
          begin
               lblProtTypeDisp.Caption := 'Start Count';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := IntToStr(Slock1.TrialPeriod);
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptDayCount:
          begin
               lblProtTypeDisp.Caption := 'Day Count';
               lblDaysLeftDisp.Caption := IntToStr(Slock1.TrialPeriod);
               lblStartsLeftDisp.Caption := 'N/A';
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := DateToStr(Slock1.ExpiryDate);
          end;
     ptExpiryDate:
          begin
               lblProtTypeDisp.Caption := 'Expiry Date';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := DateToStr(Slock1.ExpiryDate);
          end;
     ptTimed:
          begin
               lblProtTypeDisp.Caption := 'Timed';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';
               lblGraceStartsLeftDisp.Caption := 'N/A';
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptDaysUsed:
          begin
               lblProtTypeDisp.Caption := 'Days Used';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := IntToStr(Slock1.TrialPeriod);
               lblGraceStartsLeftDisp.Caption := IntToStr(Slock1.GracePeriod);
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptNone:
          begin
               lblProtTypeDisp.Caption := 'None';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';;
               lblGraceStartsLeftDisp.Caption := 'N/A';;
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     ptPK:
          begin
               lblProtTypeDisp.Caption := 'PK Override';
               lblDaysLeftDisp.Caption := 'N/A';
               lblStartsLeftDisp.Caption := 'N/A';;
               lblGraceStartsLeftDisp.Caption := 'N/A';;
               lblExpiryDateDisp.Caption := 'N/A';
          end;
     end;

     // load the registration information
     if SLock1.Registered then
     begin
          lblRegisteredToDisp.Caption := Slock1.RegName;
     end
     else
     begin
          lblRegisteredToDisp.Caption := 'N/A';
     end;

     // the serial number is a persistent field which can be set in trial mode
     lblSerialNumberDisp.Caption := Slock1.SerialNumber;

     // the user fields are persistent fields
     lblUser1.Caption := SLock1.GetUserField1;
     lblUser2.Caption := SLock1.GetUserField2;

     {set the state of the buttons/menus - these are set for this demo
     application and probably won't be the same same as for a real-life
     application}
     // the demonstration function should work until the expiry is reached
     mnmFunction.Enabled := (Slock1.Status <> stExpired);

     //The Demo function should *only* work in demo mode
     mnmDemo.Enabled := (SLock1.Status = stDemo);
end; {UpdateStatus}

{*******************************************************************************
* Procedure : btnRegisterClick                                                 *
********************************************************************************
* Purpose   : Opens up the window in which the registration information can    *
*             be entered into                                                  *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnRegisterClick(Sender: TObject);
var
     frmRegister: TfrmRegister;
begin
     // create the registration window
     frmRegister := TfrmRegister.Create(self);

     // set the caption and show the window
     frmRegister.Caption := 'Register ' + Caption;
     frmRegister.ShowModal;

     // release the window
     frmRegister.Free;

     // Update the status
     UpdateStatus;
end; {btnRegisterClick}

{*******************************************************************************
* Procedure : btnLoadCertificateClick                                          *
********************************************************************************
* Purpose   : This displays the window which accepts the certificate from the  *
*             user                                                             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnLoadCertificateClick(Sender: TObject);
begin
     // create and show the window
     frmLoadCert := TfrmLoadCert.Create(Application);
     frmLoadCert.ShowModal;
     frmLoadCert.Free;

     // Update the status
     UpdateStatus;
end; {TfrmMain.btnLoadCertificateClick}

{*******************************************************************************
* Procedure : btnExitClick                                                     *
********************************************************************************
* Purpose   : Closes the program.                                              *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.btnExitClick(Sender: TObject);
begin
     Close;
end; {btnExitClick}

{*******************************************************************************
* Procedure : Slock1CertAccept                                                 *
********************************************************************************
* Purpose   : This is triggered when a valid certificate is presented to the   *
*             application                                                      *
*             The certificate is then stored in the registry under the CertKey,*
*             CertPath and CertRoot location.                                  *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1CertAccept(Sender: TObject);
begin
     ShowMessage('The certificate has been accepted.');
     UpdateStatus;
end; {TfrmMain.Slock1CertAccept}

{*******************************************************************************
* Procedure : Slock1CertExpired                                                *
********************************************************************************
* Purpose   : This event is triggered when the expiry date of the certificate  *
*             is passed.                                                       *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1CertExpired(Sender: TObject);
begin
     ShowMessage('The certificate has expired - expiry date: ' + DateToStr(SLock1.CertExpire));
     UpdateStatus;
end; {TfrmMain.Slock1CertExpired}

{*******************************************************************************
* Procedure : Slock1CertReject                                                 *
********************************************************************************
* Purpose   : This is triggered when an invalid certificate is presented to the*
*             application                                                      *
*             The certificate is not stored in the registry                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1CertReject(Sender: TObject);
begin
     ShowMessage('The certificate was invalid and has not been accepted.');
     UpdateStatus;
end; {TfrmMain.Slock1CertReject}

{*******************************************************************************
* Procedure : Slock1ClockMovedBack                                             *
********************************************************************************
* Purpose   : People often try to get around a fixed expiry date by setting    *
*             the clock back. This event is triggered when the system clock    *
*             been set back to a date that is before the Slock was even        *
*             created. Normally you would end the program under such a         *
*             a condition.                                                     *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1ClockMovedBack(Sender: TObject);
begin
     ShowMessage('The clock was moved back!');
     UpdateStatus;
end; {Slock1ClockMovedBack}

{*******************************************************************************
* Procedure : Slock1Expired                                                    *
********************************************************************************
* Purpose   : This event is triggered when the 'CheckProtection' method is     *
*             called and the component has expired (and there is no grace      *
*             period left).                                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Expired(Sender: TObject);
begin
     ShowMessage('The program has expired. Some Functions have been disabled. Register to re-enable them.');
     UpdateStatus;
end; {Slock1Expired}

{*******************************************************************************
* Procedure : Slock1Extend                                                     *
********************************************************************************
* Purpose   : This event is triggered when the 'RegisterNow' method is called  *
*             with an extension code. The Extension code increments the        *
*             grace period.                                                    *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Extend(Sender: TObject);
begin
     ShowMessage('The component has been extended!');
     UpdateStatus;
end; {Slock1Extend}

{*******************************************************************************
* Procedure : Slock1FirstStart                                                 *
********************************************************************************
* Purpose   : This is called the first time that the Slock.CheckProtection     *
*             Method is called (for the current registry key settings). You    *
*             can use this to display a welcome message or Licensing terms and *
*             conditions or even an advert for your other software.            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1FirstStart(Sender: TObject);
begin
     ShowMessage('This is the first start! Welcome to the Slock demonstration program by TSM Inc.');
     UpdateStatus;
end; {Slock1FirstStart}

{*******************************************************************************
* Procedure : Slock1InGracePeriod                                              *
********************************************************************************
* Purpose   : This is triggered when the program has expired, but is still in  *
*             the grace period. This gives you the opportunity to display a    *
*             more urgent message to promt the user that his software will soon*
*             expire.                                                          *
*             The procedure shown here is more complcated than normally,       *
*             because we want to be able to demonstrate all the different forms*
*             of protection.                                                   *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1InGracePeriod(Sender: TObject);
begin
     if Slock1.ProtectionType = ptStartCount then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod) +
                      ' starts left in the evaluation period.');
     if Slock1.ProtectionType = ptDayCount then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod)
                       + ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptExpiryDate then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod)
                       + ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptDaysUsed then
          ShowMessage('This is the GRACE PERIOD: ' + IntToStr(Slock1.GracePeriod) +
                      ' starts left in the evaluation period.');
     UpdateStatus;
end; {Slock1InGracePeriod}

{*******************************************************************************
* Procedure : Slock1NoInitCert                                                 *
********************************************************************************
* Purpose   : This is triggered when no initial certificate is stored in the   *
*             Registry, i.e. on the first start.                               *
*             The user must enter a valid certificate before the application   *
*             can start                                                        *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1NoInitCert(Sender: TObject);
begin
     // this is a more advanced procedure for accepting certificates. For the more
     // simplified version, see the "LockTest" application.
     frmLoadInitCert := TfrmLoadInitCert.Create(Application);
     frmLoadInitCert.ShowModal;
     frmLoadInitCert.Free;
     UpdateStatus;
end; {TfrmMain.Slock1NoInitCert}

{*******************************************************************************
* Procedure : Slock1RegInfoChanged                                             *
********************************************************************************
* Purpose   : This is triggered when the registration information stored in    *
*             Windows registry is checked, and it is found that the finger-    *
*             print items no longer match the information stored in the        *
*             Certificate. The number of items which do not match is still     *
*             the level defined in the threshold, meaning that the application *
*             will still work, but will issue this message. The user should    *
*             request an updated Certificate which contains the updated        *
*             fingerprint information.                                         *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1RegInfoChanged(Sender: TObject);
begin
     ShowMessage('You have changed some of the hardware or software settings on ' +
                 'this PC. You should request an updated Certificate from the ' +
                 'software author. Please do this using the "Request Certificate" ' +
                 'button on the main Window');
     UpdateStatus;
end; {Slock1RegInfoChanged}

{*******************************************************************************
* Procedure : Slock1RegInfoTamper                                              *
********************************************************************************
* Purpose   : This event is triggered when the fingerprint information in the  *
*             Certificate does not match the information from the PC, and the  *
*             threshold has been exceeded. This means that either the PC has   *
*             been extensively altered, or someone has installed on a PC       *
*             which is not authorised.                                         *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1RegInfoTamper(Sender: TObject);
begin
     ShowMessage('The Certificate does not match this PC and LockPK cannot start.' +
                 LIT_SEP + LIT_SEP +
                 'Please contact the software author for an updated certificate. ' +
                 'You can request a certificate using the "Request Certificate" button ' +
                 'on the main window');
     UpdateStatus;
end; {Slock1RegInfoTamper}

{*******************************************************************************
* Procedure : Slock1Register                                                   *
********************************************************************************
* Purpose   : Is called when the OnRegister event in Slock is triggered. In    *
*             this example we just show a short message, but we could also     *
*             use this event to show a thankyou screen, or even enable some    *
*             additional functions.                                            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Register(Sender: TObject);
begin
     ShowMessage('The program has been registered. Thank you for registering!');
     UpdateStatus;
end; {Slock1Register}

{*******************************************************************************
* Procedure : Slock1RegWriteErr                                                *
********************************************************************************
* Purpose   : Is called when the read after write check of the Windows registry*
*             shows that the information was not correctly written.            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1RegWriteErr(Sender: TObject);
begin
     ShowMessage('There was an error writing to the registry. The program will terminate.');

     // this is a fatal error - quit
     Halt;
end; {Slock1RegWriteErr}

{*******************************************************************************
* Procedure : Slock1Reminder                                                   *
********************************************************************************
* Purpose   : This is the normal reminder event which is triggered when Slock  *
*             has not been registered and has not expired. You can use this as *
*             a reminder that the user should register.                        *
*                                                                              *
*             The procedure shown here is more complicated than normal,        *
*             because we want to be able to demonstrate all the different      *
*             forms of protection.                                             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Reminder(Sender: TObject);
begin
     if Slock1.ProtectionType = ptStartCount then
          ShowMessage('This is the normal reminder: ' + IntToStr(Slock1.TrialPeriod) +
                      ' starts left in the evaluation period.');
     if Slock1.ProtectionType = ptDayCount then
          ShowMessage('This is the normal reminder: ' + IntToStr(Trunc(Slock1.ExpiryDate - Now)) +
                      ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptExpiryDate then
          ShowMessage('This is the normal reminder: ' + IntToStr(Trunc(Slock1.ExpiryDate - Now)) +
                      ' days left in the evaluation period.');
     if Slock1.ProtectionType = ptDaysUsed then
          ShowMessage('This is the normal reminder: ' + IntToStr(Slock1.TrialPeriod) +
                      ' days left in the evaluation period.');
     UpdateStatus;
end; {Slock1Reminder}

{*******************************************************************************
* Procedure : TfrmMain.Slock1Reset                                             *
********************************************************************************
* Purpose   : Informs the user that a Reset Certificate has been received      *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Reset(Sender: TObject);
begin
     // The Reset Certificate deletes the intenal registration information and
     // resets SLockPK back to its initial state. A new Certificate is required
     // before the application can be used further
     ShowMessage('A Reset Certificate received! All SLockPK information has been deleted.');

     UpdateStatus;
end; {TfrmMain.Slock1Reset}

{*******************************************************************************
* Procedure : TfrmMain.Slock1Terminate                                         *
********************************************************************************
* Purpose   : Informs the user that a Terminate Certificate has been received  *
*             Our normal course of action is to terminate the program          *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1Terminate(Sender: TObject);
begin
     ShowMessage('Terminate certificate received. LockPK cannot ' +
                 'continue without a new valid Certificate.');

     // show the load init cert window - this window has the possibitly to either
     // accept a certificate, or to create a UIR.
     frmLoadInitCert := TfrmLoadInitCert.Create(Application);
     frmLoadInitCert.ShowModal;
     frmLoadInitCert.Free;

     // check to see if a Certificate has been loaded
     if Slock1.Status = stExpired then
     begin
          ShowMessage('No Certificate loaded. Terminating.');
          Halt;
     end; {if}
end; {TfrmMain.Slock1Terminate}

{*******************************************************************************
* Procedure : SLock1TimedOut                                                   *
********************************************************************************
* Purpose   : Is triggered when SLock times out (!! Timer mode only !!)        *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1TimedOut(Sender: TObject);
begin
     UpdateStatus;
     ShowMessage('The component has timed out!');
end; {Slock1TimedOut}

{*******************************************************************************
* Procedure : Slock1RegInfoTamper                                              *
********************************************************************************
* Purpose   : This event is triggered when the application is in Trial Mode    *
*             and there are differences between the copies of the Trial        *
*             Information - usually meaning that the user has attempted to     *
*             tamper with the trial info or has deinstalled and re-installed   *
*             an application where the DLL is being used.                      *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1TrialInfoTamper(Sender: TObject);
begin
     ShowMessage('The application has detected that there is a mismatch in the ' +
                 'internal configuration information, and the program cannot start.' +
                 LIT_SEP + LIT_SEP +
                 'Please contact the software author to remove this problem.');
end; {TfrmMain.Slock1TrialInfoTamper}

{*******************************************************************************
* Procedure : mnpNormFuncClick                                                 *
{*******************************************************************************
* Purpose   : This is an example of using Slock to control access to functions *
*             in the 'host' program. The menu point 'Function' is enabled      *
*             or disabled according to the status of Slock.Status.             *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpNormFuncClick(Sender: TObject);
begin
     ShowMessage('This is a normal function - it will be disabled after the program expired.');
end; {mnpNormFuncClick}

{*******************************************************************************
* Procedure : Slock1WrongUnlockCode                                            *
********************************************************************************
* Purpose   : This event is triggered when the user enters the wrong unlock    *
*             code. To discourage random attempts at breaking the code with    *
*             brute force, we end the program when the user enters the wrong   *
*             code.                                                            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.Slock1WrongUnlockCode(Sender: TObject);
begin
     ShowMessage('The unlock code was wrong. Program was not registered and will end.');
     Halt;
end; {Slock1WrongUnlockCode}

{*******************************************************************************
* Procedure : mnpExitClick                                                     *
********************************************************************************
* Purpose   : Closes the program.                                              *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpExitClick(Sender: TObject);
begin
     Close;
end; {mnpExitClick}

{*******************************************************************************
* Procedure : mnpRegNowClick                                                   *
********************************************************************************
* Purpose   : This calls event 'btnRegisterClick' to register Slock            *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpRegNowClick(Sender: TObject);
begin
     btnRegisterClick(self);
end; {mnpRegNowClick}

{*******************************************************************************
* Procedure : mnpLoadCertificateClick                                          *
********************************************************************************
* Purpose   : This calls event 'btnLoadCertificate' to register Slock          *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpLoadCertificateClick(Sender: TObject);
begin
     btnLoadCertificateClick(self);
end; {TfrmMain.mnpLoadCertificateClick}

{*******************************************************************************
* Procedure : TfrmMain.mnpDemoFunctionClick                                    *
********************************************************************************
* Purpose   : This is a function which is only available in Demo mode          *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpDemoFunctionClick(Sender: TObject);
begin
     ShowMessage('This function is only available in demo mode');
end; {TfrmMain.mnpDemoFunctionClick}

{*******************************************************************************
* Procedure : TfrmMain.mnpGettingHelpClick                                     *
********************************************************************************
* Purpose   : This is a function which is only available in Demo mode          *
********************************************************************************
* Paramters : None                                                             *
********************************************************************************
* Returns   : None                                                             *
*******************************************************************************}
procedure TfrmMain.mnpGettingHelpClick(Sender: TObject);
begin
     ShowMessage('Please see the section "Example - LockPK" in the index of the ' +
                 'SLock help file.');
end; {TfrmMain.mnpGettingHelpClick}

end.
